/*
    Copyright (c) 2005 Peter Persson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
    Changelog
    2005-11-20	Created (Peter Persson)
    2005-12-03 Misc. minor changes (Peter Persson)
    2006-01-15 Rewrote main emulation loop (Peter Persson)
*/

#include "osd.h"

/* Options */
int frameskip;
int limitspeed;

/* Display timing */
struct timeval tv_start,tv;
double elapsed_time;
double speed_index;

/* Game file name */
char game_name[0x100];

/* pointer to screen buffer (not necessarily physical) */
unsigned char* vscreen;


int main(int argc, char **argv)
{

	int i;
	
	/* Print help if no game specified */
	if(argc < 2)
	{
		printf("\n%s\n", APP_NAME);
		printf("Copyright (C) Charles MacDonald 1998-2003\n");
		printf("Version %s, build date: %s, %s\n", APP_VERSION, __DATE__, __TIME__);
		printf("F030 port by Peter Persson (pedda_pirat@hotmail.com)\n");
		printf("IKBD stuff by Patrice Mandin\n");		
		printf("C2P stuff by Mikael Kalms\n\n");		

		printf("Usage: sp.ttp <filename.ext> [-options]\n");
		printf("Type 'sp.ttp -help' for a summary of the available options\n");
		exit(1);
	}
	
	/* Show option list */
	if(stricmp(argv[1], "-help") == 0)
	{
		printf("Options:\n");
		printf(" -jp          \t use Japanese console type.\n");
		printf(" -fskip <n>   \t specify frameskip value (default =1 = no skip).\n");
		exit(1);
	}
	
	/* set defaults */
	sms.use_fm = 0;
	sms.save = 0;
	sms.country = TYPE_EXPORT;
	snd.enabled = 1;
	memset(&bitmap, 0, sizeof(t_bitmap));
	bitmap.depth = 8;
	
	/* Parse options */
	for(i = 0; i < argc; i++)
	{
		int left = argc - i - 1;

		if(stricmp(argv[i], "-fskip") == 0 && left)
		{
			frameskip = atoi(argv[i+1]);
			if(frameskip<1) frameskip = 1;
		}
	
		if(stricmp(argv[i], "-bpp") == 0 && left)
		{
			bitmap.depth = atoi(argv[i+1]);
		}

		if(stricmp(argv[i], "-jp") == 0)
		{
			sms.country = TYPE_DOMESTIC;
		}
	}

	/* initialise screen stuff */

	switch(bitmap.depth)
		{
			case 8:
				frameskip = 1;
				Open_screendriver = &OpenVIDEL5;
				Close_screendriver = &CloseVIDEL5;
				Update_screendriver = &UpdateVIDEL5;
				Palette_screendriver = &PaletteVIDEL5;
				bitmap.width  = 256;
				bitmap.pitch  = 256;
				bitmap.height = 192;
				bitmap.depth  = 8;
				break;

			case 16:
				frameskip = 2;
				Open_screendriver = &OpenVIDEL16;
				Close_screendriver = &CloseVIDEL16;
				Update_screendriver = &UpdateVIDEL16;
				Palette_screendriver = &PaletteVIDEL16;
				bitmap.width  = 256;
				bitmap.pitch  = 512;
				bitmap.height = 192;
				bitmap.depth  = 16;
				break;
			default:
				printf("Error, illegal -bpp <value>.\n");
				exit(1);
		}		

	vscreen = (unsigned char*) Open_screendriver();
	bitmap.data   = (unsigned char*)vscreen;
	bitmap.viewport.x = 0;
	bitmap.viewport.y = 0;
	bitmap.viewport.w = bitmap.width;
	bitmap.viewport.h = bitmap.height;

	/* Make copy of game filename */
	strcpy(game_name, argv[1]);

	/* Attempt to load game off commandline */
	if(load_rom(game_name) == 0)
	{
		printf("Error loading `%s'.\n", game_name);
		exit(1);
	}

	/* Set up video, audio, input, etc. */
	Supexec(PSG_Open);
	input_open();

	/* Initialize the virtual console emulation */
	system_init(0);

	/* Load SRAM from disk if possible */
	load_sram();

	/* Call emulation loop */
	Supexec(run_emulation);
	
	/* Done. Save SRAM and restore video, audio, input etc. */
	save_sram();

	input_close();
	Supexec(PSG_Close);
	Close_screendriver();
	Mfree( vscreen );

	return 0;
}


void run_emulation(void)
{
	int framecounter=frameskip-1;
	
	while(!input_poll())
	{
	
		if(framecounter)
		{
			system_frame(1);
			framecounter--;
		} else
		{
			system_frame(0);
			if(bitmap.pal.update)
				Palette_screendriver((unsigned char*)&bitmap.pal.color);
				
			Update_screendriver(vscreen,frameskip);

			framecounter = frameskip-1;
		}
	}
}


void system_load_sram(void)
{
	load_sram();
}

/* Load SRAM data */
void load_sram(void)
{
    char name[0x100];
    FILE *fd;
    strcpy(name, game_name);
    strcpy(strrchr(name, '.'), ".sav");
    fd = fopen(name, "rb");
    if(fd)
    {
        sms.save = 1;
        fread(sms.sram, 0x8000, 1, fd);
        fclose(fd);
    }
}


/* Save SRAM data */
void save_sram(void)
{
    if(sms.save)
    {
        char name[0x100];
        FILE *fd = NULL;
        strcpy(name, game_name);
        strcpy(strrchr(name, '.'), ".sav");
        fd = fopen(name, "wb");
        if(fd)
        {
            fwrite(sms.sram, 0x8000, 1, fd);
            fclose(fd);
        }
    }
}

